class AccessorsMakeFinal1 {
	private String test;
	/**
	 * @return {@code this}.
	 */
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public final AccessorsMakeFinal1 test(final String test) {
		this.test = test;
		return this;
	}
}
