import lombok.val;
public class ValInBasicFor {
  public ValInBasicFor() {
    super();
  }
  public void basicFor() {
    java.util.List<String> list = java.util.Arrays.asList("Hello, World!");
    for (val shouldBe = 1;, val marked = "";, val error = 1.0;; ; ) 
      {
        System.out.println("");
      }
  }
}