class AccessorsConfiguration {
	private String m_FieldName = "";
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public String fieldName() {
		return this.m_FieldName;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public void fieldName(final String m_FieldName) {
		this.m_FieldName = m_FieldName;
	}
}
class AccessorsConfiguration2 {
	private String m_FieldName = "";
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public void setM_FieldName(final String m_FieldName) {
		this.m_FieldName = m_FieldName;
	}
}
class AccessorsConfiguration3 {
	private String fFieldName = "";
	/**
	 * @return {@code this}.
	 */
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public AccessorsConfiguration3 setFieldName(final String fFieldName) {
		this.fFieldName = fFieldName;
		return this;
	}
}
