/*
 * Copyright (C) 2009 The Project Lombok Authors.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <jni.h>
#include <windows.h>
#include <stdio.h>
#include <winbase.h>
#include "lombok_installer_WindowsDriveInfo.h"

/*
 * Class:     lombok_installer_WindowsDriveInfo
 * Method:    getLogicalDrives0
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_lombok_installer_WindowsDriveInfo_getLogicalDrives0
  (JNIEnv *env, jobject obj) {
	return GetLogicalDrives();
}

/*
 * Class:     lombok_installer_WindowsDriveInfo
 * Method:    getDriveType
 * Signature: (Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_lombok_installer_WindowsDriveInfo_getDriveType
  (JNIEnv *env, jobject obj, jstring drive) {
	const char *str= (*env)->GetStringUTFChars(env, drive, 0);
	DWORD val = GetDriveTypeA(str);
	(*env)->ReleaseStringUTFChars(env, drive, str);
	return val;
}
