class SetterDeprecated {
	@Deprecated
	int annotation;
	/**
	 * @deprecated
	 */
	int javadoc;
	@java.lang.Deprecated
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public void setAnnotation(final int annotation) {
		this.annotation = annotation;
	}
	/**
	 * @deprecated
	 */
	@java.lang.Deprecated
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public void setJavadoc(final int javadoc) {
		this.javadoc = javadoc;
	}
}
