class GenerateSuppressFBWarnings {
  @lombok.Getter int y;
  GenerateSuppressFBWarnings() {
    super();
  }
  public @java.lang.SuppressWarnings("all") @edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code") @lombok.Generated int getY() {
    return this.y;
  }
}
