@lombok.RequiredArgsConstructor class FieldDefaultsViaConfigAndRequiredArgsConstructor {
  final int x;
  public @java.lang.SuppressWarnings("all") @lombok.Generated FieldDefaultsViaConfigAndRequiredArgsConstructor(final int x) {
    super();
    this.x = x;
  }
}
