abstract class WithMethodAbstract {
	String foo;
	/**
	 * @return a clone of this object, except with this updated property (returns {@code this} if an identical value is passed).
	 */
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public abstract WithMethodAbstract withFoo(final String foo);
}
