package pkg;

public interface Interface {

	boolean equals(java.lang.Object o);

	boolean canEqual(java.lang.Object other);

	int hashCode();

	String getString();

	int getInteger();

	void setString(String string);

	void setInteger(int integer);

	java.lang.String toString();

}