/*
 * These are stub versions of various bits of javac-internal API (for various different versions of javac). Lombok is compiled against these.
 */
package com.sun.tools.javac.parser;

import com.sun.tools.javac.util.Context;

public class ParserFactory {
	public static Context.Key<ParserFactory> parserFactoryKey;
	
	public ParserFactory(Context context) {
		
	}
}