/*
 * These are stub versions of various bits of javac-internal API (for various different versions of javac). Lombok is compiled against these.
 */
package com.sun.tools.javac.main;


public enum Option {
	;
	public String text;
	public String primaryName;
}
