package lombok.permit.dummy;

@SuppressWarnings("all")
public final class GrandChild extends Child {
	private Class<?> a;
	private int b;
	private String c;
	private Class<?> d;
	private Class<?>[] e;
	private Class<?>[] f;
	private int g;
	private transient String h;
	private transient Object i;
	private byte[] j;
	private byte[] k;
	private byte[] l;
	private volatile Object m;
	private Object n;
}
