class UtilityClassInner {
  static final @lombok.experimental.UtilityClass class UtilClass {
    private @java.lang.SuppressWarnings("all") @lombok.Generated UtilClass() {
      super();
      throw new java.lang.UnsupportedOperationException("This is a utility class and cannot be instantiated");
    }
  }
  UtilityClassInner() {
    super();
  }
}
