/**
 * This is a reduced copy of the nqzero Permit-reflect project.
 * https://github.com/nqzero/permit-reflect
 * 
 * Many thanks to nqzero. The permit-reflect project is, like lombok itself, licensed under the MIT license.
 * See https://github.com/nqzero/permit-reflect/blob/master/License for license info.
 */
package lombok.permit;