class GetterAccessLevel {
	boolean isNone;
	boolean isPrivate;
	boolean isPackage;
	boolean isProtected;
	boolean isPublic;
	String noneString;
	String privateString;
	String packageString;
	String protectedString;
	String publicString;
	String value;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	private boolean isPrivate() {
		return this.isPrivate;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	boolean isPackage() {
		return this.isPackage;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	protected boolean isProtected() {
		return this.isProtected;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public boolean isPublic() {
		return this.isPublic;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	private String getPrivateString() {
		return this.privateString;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	String getPackageString() {
		return this.packageString;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	protected String getProtectedString() {
		return this.protectedString;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public String getPublicString() {
		return this.publicString;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public String getValue() {
		return this.value;
	}
}
