class UtilityClassInner {
	static final class UtilClass {
		@java.lang.SuppressWarnings("all")
		@lombok.Generated
		private UtilClass() {
			throw new java.lang.UnsupportedOperationException("This is a utility class and cannot be instantiated");
		}
	}
}
