// version 14:
public record FieldNameConstantsOnRecord(String iAmADvdPlayer, int $skipMe, int andMe, String butPrintMePlease) {
	static double skipMeToo;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	static final class Fields {
		public static final java.lang.String iAmADvdPlayer = "iAmADvdPlayer";
		public static final java.lang.String butPrintMePlease = "butPrintMePlease";
	}
}
