class GetterPlain {
	int i;
	int foo;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public int getI() {
		return this.i;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public int getFoo() {
		return this.foo;
	}
}
