class AccessorsNoParams {
	private String otherFieldWithOverride = "";
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public String otherFieldWithOverride() {
		return this.otherFieldWithOverride;
	}
}
class AccessorsNoParams2 {
	private boolean foo;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public void setFoo(final boolean foo) {
		this.foo = foo;
	}
}
