class GetterWithJavaBeansSpecCapitalization {
	int a;
	int aField;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public int getA() {
		return this.a;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public int getaField() {
		return this.aField;
	}
}
