class MultiFieldGetter {
	int x;
	int y;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	protected int getX() {
		return this.x;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	protected int getY() {
		return this.y;
	}
}
class MultiFieldGetter2 {
	int x;
	int y;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	int getX() {
		return this.x;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	int getY() {
		return this.y;
	}
}
