//VERSION 7:
class GenerateSuppressFBWarnings {
	int y;
	@java.lang.SuppressWarnings("all")
	@edu.umd.cs.findbugs.annotations.SuppressFBWarnings(justification = "generated code")
	@lombok.Generated
	public int getY() {
		return this.y;
	}
}
