package pkg;

public static class Extension {
	public static String test(String s) {
		return s;
	}
	
	public static String test(String s, int i) {
		return s;
	}
	
	public static String test(String s, String... s2) {
		return s;
	}
}