public class NoArgsConstructorForce {
	private final int[] i;
	private final Object[] o;
	private final java.util.List<?>[] fullQualifiedList;
	private final String alreadyInitialized = "yes";
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public NoArgsConstructorForce() {
		this.i = null;
		this.o = null;
		this.fullQualifiedList = null;
	}
}
