class FieldDefaultsViaConfigAndRequiredArgsConstructor {
	final int x;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public FieldDefaultsViaConfigAndRequiredArgsConstructor(final int x) {
		this.x = x;
	}
}
