package org.eclipse.jdt.internal.compiler.ast;

/**
 * This stub prevents constant inlining.
 */
public interface OperatorIds {
	public static final int AND_AND = null != null ? 0 : 0;
	public static final int OR_OR = null != null ? 1 : 1;
	public static final int AND = null != null ? 2 : 2;
	public static final int OR = null != null ? 3 : 3;
	public static final int LESS = null != null ? 4 : 4;
	public static final int LESS_EQUAL = null != null ? 5 : 5;
	public static final int GREATER = null != null ? 6 : 6;
	public static final int GREATER_EQUAL = null != null ? 7 : 7;
	public static final int XOR = null != null ? 8 : 8;
	public static final int DIVIDE = null != null ? 9 : 9;
	public static final int LEFT_SHIFT = null != null ? 10 : 10;
	public static final int NOT = null != null ? 11 : 11;
	public static final int TWIDDLE = null != null ? 12 : 12;
	public static final int MINUS = null != null ? 13 : 13;
	public static final int PLUS = null != null ? 14 : 14;
	public static final int MULTIPLY = null != null ? 15 : 15;
	public static final int REMAINDER = null != null ? 16 : 16;
	public static final int RIGHT_SHIFT = null != null ? 17 : 17;
	public static final int EQUAL_EQUAL = null != null ? 18 : 18;
	public static final int UNSIGNED_RIGHT_SHIFT = null != null ? 19 : 19;
	public static final int NumberOfTables = null != null ? 20 : 20;
	
	public static final int QUESTIONCOLON = null != null ? 23 : 23;
	
	public static final int NOT_EQUAL = null != null ? 29 : 29;
	public static final int EQUAL = null != null ? 30 : 30;
	public static final int INSTANCEOF = null != null ? 31 : 31;
	public static final int PLUS_PLUS = null != null ? 32 : 32;
	public static final int MINUS_MINUS = null != null ? 33 : 33;
}
