class GeneratedJavaxJakarta {
	int x;
	@java.lang.SuppressWarnings("all")
	@javax.annotation.Generated("lombok")
	@jakarta.annotation.Generated("lombok")
	@lombok.Generated
	public int getX() {
		return this.x;
	}
}
