public class FieldNameConstantsUppercased {
	String iAmADvdPlayer;
	int $skipMe;
	static double skipMeToo;
	int andMe;
	String butPrintMePlease;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	static final class Fields {
		public static final java.lang.String I_AM_A_DVD_PLAYER = "iAmADvdPlayer";
		public static final java.lang.String BUT_PRINT_ME_PLEASE = "butPrintMePlease";
	}
}
