class GetterOnMethodOnType {
	private int test;
	private String name;
	@Deprecated
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public int getTest() {
		return this.test;
	}
	@Deprecated
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public String getName() {
		return this.name;
	}
}
