class ExtensionMethodNonStatic {
	public void test() {
		String s = "test";
		s.startsWith("");
	}

	static class Extensions {
		public boolean startsWith(String s, String prefix) {
			return s.startsWith(prefix);
		}
	}
}