class ConstructorInner {
	static class Inner {
		@java.lang.SuppressWarnings("all")
		@lombok.Generated
		private Inner() {
		}
		@java.lang.SuppressWarnings("all")
		@lombok.Generated
		public static ConstructorInner.Inner of() {
			return new ConstructorInner.Inner();
		}
	}
}
