public class FieldNameConstantsConfigKeys {
	String iAmADvdPlayer;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public static final class Foobar {
		public static final java.lang.String iAmADvdPlayer = "iAmADvdPlayer";
	}
}
