/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class lombok_installer_WindowsDriveInfo */

#ifndef _Included_lombok_installer_WindowsDriveInfo
#define _Included_lombok_installer_WindowsDriveInfo
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     lombok_installer_WindowsDriveInfo
 * Method:    getLogicalDrives0
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_lombok_installer_WindowsDriveInfo_getLogicalDrives0
  (JNIEnv *, jobject);

/*
 * Class:     lombok_installer_WindowsDriveInfo
 * Method:    getDriveType
 * Signature: (Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_lombok_installer_WindowsDriveInfo_getDriveType
  (JNIEnv *, jobject, jstring);

#ifdef __cplusplus
}
#endif
#endif
