import lombok.With;
public record WithOnNestedRecord()<T> {
  public @With record Nested(String a, String b) {
/* Implicit */    private final String a;
/* Implicit */    private final String b;
    /**
     * @return a clone of this object, except with this updated property (returns {@code this} if an identical value is passed).
     */
    public @java.lang.SuppressWarnings("all") @lombok.Generated WithOnNestedRecord.Nested withA(final String a) {
      return ((this.a == a) ? this : new WithOnNestedRecord.Nested(a, this.b));
    }
    /**
     * @return a clone of this object, except with this updated property (returns {@code this} if an identical value is passed).
     */
    public @java.lang.SuppressWarnings("all") @lombok.Generated WithOnNestedRecord.Nested withB(final String b) {
      return ((this.b == b) ? this : new WithOnNestedRecord.Nested(this.a, b));
    }
  }
}
