class GeneratedJavaxOnLombokOn {
  @lombok.Getter int x;
  GeneratedJavaxOnLombokOn() {
    super();
  }
  public @java.lang.SuppressWarnings("all") @javax.annotation.Generated("lombok") @lombok.Generated int getX() {
    return this.x;
  }
}
