import lombok.extern.slf4j.Slf4j;
public @Slf4j record LoggerSlf4jOnRecord(String a, String b) {
/* Implicit */  private final String a;
/* Implicit */  private final String b;
  private static final @java.lang.SuppressWarnings("all") @lombok.Generated org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(LoggerSlf4jOnRecord.class);
  <clinit>() {
  }
}
