//CONF: lombok.anyConstructor.suppressConstructorProperties = true
//CONF: lombok.toString.doNotUseGetters = true
//CONF: lombok.equalsAndHashCode.doNotUseGetters = true
//CONF: lombok.noArgsConstructor.extraPrivate = true
@lombok.Data
class DataConfiguration {
	final int x;
}
