public @lombok.experimental.FieldNameConstants class FieldNameConstantsConfigKeys {
  public static final @java.lang.SuppressWarnings("all") @lombok.Generated class Foobar {
    public static final java.lang.String iAmADvdPlayer = "iAmADvdPlayer";
    <clinit>() {
    }
    private @java.lang.SuppressWarnings("all") @lombok.Generated Foobar() {
      super();
    }
  }
  String iAmADvdPlayer;
  public FieldNameConstantsConfigKeys() {
    super();
  }
}
