class SetterPlain {
	int i;
	int foo;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public void setI(final int i) {
		this.i = i;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public void setFoo(final int foo) {
		this.foo = foo;
	}
}
