class SetterWithJavaBeansSpecCapitalization {
	int a;
	int aField;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public void setA(final int a) {
		this.a = a;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public void setaField(final int aField) {
		this.aField = aField;
	}
}
