import lombok.NoArgsConstructor;

@NoArgsConstructor(force = true)
public class NoArgsConstructorForce {
	private final int[] i;
	private final Object[] o;
	private final java.util.List<?>[] fullQualifiedList;
	private final String alreadyInitialized = "yes";
}