//CONF: lombok.addNullAnnotations = javax
import javax.annotation.ParametersAreNonnullByDefault;
@lombok.EqualsAndHashCode
@ParametersAreNonnullByDefault
class EqualsAndHashCodeWithNonNullByDefault {
}