public class FieldNameConstantsEnum {
	String iAmADvdPlayer;
	int $dontSkipMe;
	static double alsoDontSkipMe;
	int butSkipMe;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public enum TypeTest {
		iAmADvdPlayer, $dontSkipMe, alsoDontSkipMe;
	}
}
