class AccessorsMakeFinalLombokConfig {
  private @lombok.Setter String test;
  AccessorsMakeFinalLombokConfig() {
    super();
  }
  public final @java.lang.SuppressWarnings("all") @lombok.Generated void setTest(final String test) {
    this.test = test;
  }
}
