class Getter {
	static boolean foo;
	static int bar;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public static boolean isFoo() {
		return Getter.foo;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public static int getBar() {
		return Getter.bar;
	}
}
