class ToStringExplicitIncludeConf {
	int x;
	int y;
	@java.lang.Override
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public java.lang.String toString() {
		return "ToStringExplicitIncludeConf(y=" + this.y + ")";
	}
}
