@lombok.ToString(onlyExplicitlyIncluded = true) class ToStringExplicitInclude {
  int x;
  ToStringExplicitInclude() {
    super();
  }
  public @java.lang.Override @java.lang.SuppressWarnings("all") @lombok.Generated java.lang.String toString() {
    return "ToStringExplicitInclude()";
  }
}
