class SetterAccessLevel {
	boolean isNone;
	boolean isPrivate;
	boolean isPackage;
	boolean isProtected;
	boolean isPublic;
	boolean value;
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	private void setPrivate(final boolean isPrivate) {
		this.isPrivate = isPrivate;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	void setPackage(final boolean isPackage) {
		this.isPackage = isPackage;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	protected void setProtected(final boolean isProtected) {
		this.isProtected = isProtected;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public void setPublic(final boolean isPublic) {
		this.isPublic = isPublic;
	}
	@java.lang.SuppressWarnings("all")
	@lombok.Generated
	public void setValue(final boolean value) {
		this.value = value;
	}
}
