import lombok.AllArgsConstructor;

@AllArgsConstructor
public class ConstructorJavadoc {
	/**
	 * Some text
	 *
	 * @param fieldName Hello, World1
	 * --- GETTER ---
	 * Getter section
	 *
	 * @return Sky is blue1
	 */
	private int fieldName;
	
	/**
	 * Sky is blue
	 */
	private int fieldName2;
}
